@@ -1,24 +1,24 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2012 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2015 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
  *   Free Software Foundation; either version 2 of the License, or (at your
  *   option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *   See the GNU General Public License for more details.
- * 
+ *
  *   You should have received a copy of the GNU General Public License along
  *   with this program; if not, see <http://www.gnu.org/licenses>.
- * 
+ *
  *   Linking stunnel statically or dynamically with other modules is making
  *   a combined work based on stunnel. Thus, the terms and conditions of
  *   the GNU General Public License cover the whole combination.
- * 
+ *
  *   In addition, as a special exception, the copyright holder of stunnel
  *   gives you permission to combine stunnel with free software programs or
  *   libraries that are released under the GNU LGPL and with code included
@@ -26,7 +26,7 @@
  *   modified versions of such code, with unchanged license). You may copy
  *   and distribute such a system following the terms of the GNU GPL for
  *   stunnel and the licenses of the other code concerned.
- * 
+ *
  *   Note that people who make modified versions of stunnel are not obligated
  *   to grant this special exception for their modified versions; it is their
  *   choice whether to do so. The GNU General Public License gives permission
@@ -39,20 +39,23 @@
 #include "prototypes.h"
 
     /* global OpenSSL initalization: compression, engine, entropy */
-static int init_compression(GLOBAL_OPTIONS *);
-static int init_prng(GLOBAL_OPTIONS *);
-static int add_rand_file(GLOBAL_OPTIONS *, const char *);
+NOEXPORT int compression_init(GLOBAL_OPTIONS *);
+NOEXPORT int prng_init(GLOBAL_OPTIONS *);
+NOEXPORT int add_rand_file(GLOBAL_OPTIONS *, const char *);
 
 int cli_index, opt_index; /* to keep structure for callbacks */
 
-void ssl_init(void) { /* init SSL before parsing configuration file */
+int ssl_init(void) { /* init SSL before parsing configuration file */
     SSL_load_error_strings();
     SSL_library_init();
     cli_index=SSL_get_ex_new_index(0, "cli index", NULL, NULL, NULL);
     opt_index=SSL_CTX_get_ex_new_index(0, "opt index", NULL, NULL, NULL);
-#ifdef HAVE_OSSL_ENGINE_H
+    if(cli_index<0 || opt_index<0)
+        return 1;
+#ifndef OPENSSL_NO_ENGINE
     ENGINE_load_builtin_engines();
 #endif
+    return 0;
 }
 
 int ssl_configure(GLOBAL_OPTIONS *global) { /* configure global SSL settings */
@@ -65,26 +68,33 @@ int ssl_configure(GLOBAL_OPTIONS *global) { /* configure global SSL settings */
             return 1;
         }
     }
-    s_log(LOG_NOTICE, "FIPS mode is %s",
+    s_log(LOG_NOTICE, "FIPS mode %s",
         global->option.fips ? "enabled" : "disabled");
 #endif /* USE_FIPS */
-    if(init_compression(global))
+#ifndef OPENSSL_NO_COMP
+    if(compression_init(global))
         return 1;
-    if(init_prng(global))
+#endif /* OPENSSL_NO_COMP */
+    if(prng_init(global))
         return 1;
     s_log(LOG_DEBUG, "PRNG seeded successfully");
     return 0; /* SUCCESS */
 }
 
-static int init_compression(GLOBAL_OPTIONS *global) {
 #ifndef OPENSSL_NO_COMP
+NOEXPORT int compression_init(GLOBAL_OPTIONS *global) {
     SSL_COMP *comp;
     STACK_OF(SSL_COMP) *ssl_comp_methods;
 
     ssl_comp_methods=SSL_COMP_get_compression_methods();
     if(!ssl_comp_methods) {
-        s_log(LOG_ERR, "Failed to get compression methods");
-        return 1;
+        if(global->compression==COMP_NONE) {
+            s_log(LOG_NOTICE, "Failed to get compression methods");
+            return 0; /* ignore */
+        } else {
+            s_log(LOG_ERR, "Failed to get compression methods");
+            return 1;
+        }
     }
 
     /* delete OpenSSL defaults (empty the SSL_COMP stack) */
@@ -93,11 +103,11 @@ static int init_compression(GLOBAL_OPTIONS *global) {
         OPENSSL_free(sk_SSL_COMP_pop(ssl_comp_methods));
 
     if(global->compression==COMP_NONE) {
-        s_log(LOG_DEBUG, "Compression not enabled");
+        s_log(LOG_DEBUG, "Compression disabled");
         return 0; /* success */
     }
 
-    /* insert RFC 1951 (DEFLATE) algoritm */
+    /* insert RFC 1951 (DEFLATE) algorithm */
     if(SSLeay()>=0x00908051L) { /* 0.9.8e-beta1 */
         /* only allow DEFLATE with OpenSSL 0.9.8 or later
            with openssl #1468 zlib memory leak fixed */
@@ -113,11 +123,11 @@ static int init_compression(GLOBAL_OPTIONS *global) {
             s_log(LOG_ERR, "Failed to initialize compression method");
             return 1;
         }
-        comp->name=comp->method->name;
+        comp->name=(char *)(comp->method->name);
         sk_SSL_COMP_push(ssl_comp_methods, comp);
     }
 
-    /* also insert one of obsolete (ZLIB/RLE) algoritms */
+    /* also insert one of obsolete (ZLIB/RLE) algorithms */
     comp=(SSL_COMP *)OPENSSL_malloc(sizeof(SSL_COMP));
     if(!comp) {
         s_log(LOG_ERR, "OPENSSL_malloc filed");
@@ -140,20 +150,20 @@ static int init_compression(GLOBAL_OPTIONS *global) {
         s_log(LOG_ERR, "Failed to initialize compression method");
         return 1;
     }
-    comp->name=comp->method->name;
+    comp->name=(char *)(comp->method->name);
     sk_SSL_COMP_push(ssl_comp_methods, comp);
     s_log(LOG_INFO, "Compression enabled: %d algorithm(s)",
         sk_SSL_COMP_num(ssl_comp_methods));
-#endif /* OPENSSL_NO_COMP */
     return 0; /* success */
 }
+#endif /* OPENSSL_NO_COMP */
 
-static int init_prng(GLOBAL_OPTIONS *global) {
+NOEXPORT int prng_init(GLOBAL_OPTIONS *global) {
     int totbytes=0;
     char filename[256];
+#ifndef USE_WIN32
     int bytes;
-
-    bytes=0; /* avoid warning if #ifdef'd out for windows */
+#endif
 
     filename[0]='\0';
 
@@ -199,12 +209,11 @@ static int init_prng(GLOBAL_OPTIONS *global) {
                          so no need to check if seeded sufficiently */
         }
     }
-#endif /* USE_WIN32 */
-
     /* try the good-old default /dev/urandom, if available  */
     totbytes+=add_rand_file(global, "/dev/urandom");
     if(RAND_status())
         return 0; /* success */
+#endif /* USE_WIN32 */
 
     /* random file specified during configure */
     s_log(LOG_ERR, "PRNG seeded with %d bytes total", totbytes);
@@ -212,21 +221,21 @@ static int init_prng(GLOBAL_OPTIONS *global) {
     return 1; /* FAILED */
 }
 
-static int add_rand_file(GLOBAL_OPTIONS *global, const char *filename) {
+NOEXPORT int add_rand_file(GLOBAL_OPTIONS *global, const char *filename) {
     int readbytes;
     int writebytes;
     struct stat sb;
 
     if(stat(filename, &sb))
-        return 0; /* could not stat() file  -> return 0 bytes */
+        return 0; /* could not stat() file --> return 0 bytes */
     if((readbytes=RAND_load_file(filename, global->random_bytes)))
         s_log(LOG_DEBUG, "Snagged %d random bytes from %s",
             readbytes, filename);
     else
-        s_log(LOG_INFO, "Unable to retrieve any random data from %s",
+        s_log(LOG_INFO, "Cannot retrieve any random data from %s",
             filename);
     /* write new random data for future seeding if it's a regular file */
-    if(global->option.rand_write && (sb.st_mode & S_IFREG)){
+    if(global->option.rand_write && (sb.st_mode & S_IFREG)) {
         writebytes=RAND_write_file(filename);
         if(writebytes==-1)
             s_log(LOG_WARNING, "Failed to write strong random data to %s - "
